//#define DEBUG

//#define TRACE bl trace
#define TRACE

#ifdef DEBUG
.macro	ERROR	code
	bkpt	#\code
.endm
.macro	CERROR	cond, code
	b\cond	.+8
	b	.+8
	bkpt	#\code
.endm
#else
.macro	ERROR	code
	b	exit_emulator
.endm
.macro	CERROR	cond, code
	b\cond	exit_emulator
.endm
#endif

// System calls
#define e_touchpad_read 75
#define e_touchpad_write 76

// Global data structure: r9 points to this at all times
// Note some requirements:
// * to do "ldr reg, [r9, #s_xxx]", offset must be below 0x1000
// * to do "add reg, r9, #s_xxx",   offset must be representable as a shifted byte
// When changing an existing offset, don't forget to "del *.o"!

#define s_wram                  0x0000  // 0800 bytes

// CPU data
#define s_mem_map               0x0800  // 0024 bytes (9 entries, 4 bytes each)
#define s_flags_di              0x0824  // 0004 bytes
#define s_pc_base               0x0828  // 0004 bytes
#define s_interrupts            0x082C
#define s_irq_from_apu          0x082C
#define s_irq_from_mapper       0x082D
#define s_nmi_reset             0x082F  // set to FF for reset, 80 for nmi

// Emulation data (no relation to anything in an actual NES)
#define s_frame_count           0x0840
#define s_frame_count_rtc       0x0844
#define s_saved_sp              0x0848
#define s_frame_timer           0x084C
#define s_touchpad_size         0x0850
#define s_keypad_command_map    0x0854
#define s_keypad_update_input   0x0858
#define s_command_keys_pressed  0x085C
#define s_saved_irq_mask        0x0860
#define s_saved_irq_handler     0x0864
#define s_frameskip             0x0868
#define s_frameskip_cur         0x086C
#define s_spr_loc_table_valid   0x0870

#define s_input_status          0x08C0
#define s_input_queue           0x08C4

// PPU ("Picture Processing Unit") data
#define s_ppu_oam_ram           0x0900  // 0100 bytes - must be 256-byte aligned
#define s_ppu_mem_map           0x0A00  // 0040 bytes
#define s_ppu_palette_gray      0x0A40  // 0040 bytes
#define s_ppu_palette           0x0A80  // 0020 bytes
#define s_ppu_flags             0x0AA0
#define s_ppu_control           s_ppu_flags+0   // $2000
#define s_ppu_mask              s_ppu_flags+1   // $2001
#define s_ppu_status            s_ppu_flags+2   // $2002
#define s_ppu_oam_addr          0x0AA4          // $2003
#define s_ppu_scroll            0x0AA8          // $2005 (x/y toggle in bit 16, x fine in bits 29-31)
#define s_ppu_address           0x0AAC          // $2006
#define s_ppu_data              0x0AB0          // $2007 buffer
#define s_ppu_scanline          0x0AB4

// ROM data
#define s_rom_header            0x0B00
#define s_mapper                0x0B0C
#define s_prg_size              0x0B10
#define s_prg_ptr               0x0B14
#define s_chr_size              0x0B18
#define s_chr_ptr               0x0B1C

#define s_mapper_state          0x0B40  // 0010 bytes

// The big stuff
#define s_spr_loc_table         0x0B90  // 0870 bytes (9 * 240)
#define s_name_table_ram        0x1400  // 1000 bytes - must be 128-byte aligned
#define s_chr_ram               0x2400  // 2000 bytes
#define s_sram                  0x4400  // 2000 bytes

#define s_SIZE                  0x6400
#define s_ALIGN                 0x0100

// CPU register usage:
// r0 = general purpose temporary
// r1 = next instruction byte
// r2 = address low byte or full address
// r3 = address high byte
#define cpu_pc          r4
#define cpu_cycles      r5
#define cpu_a           r6
#define cpu_x           r7
#define cpu_y           r8
#define cpu_sp          r10
#define cpu_flags       r11
#define cpu_itable      r12

#define CPU_CYCLE_LENGTH 3
